@@ -9,7 +9,7 @@
 
    This file is part of LibTMCG.
 
- Copyright (C) 2004, 2005, 2006, 2007  Heiko Stamer <stamer@gaos.org>
+ Copyright (C) 2004, 2005, 2006, 2007, 2009  Heiko Stamer <stamer@gaos.org>
 
    LibTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -157,12 +157,8 @@ bool BarnettSmartVTMF_dlog::CheckElement
 	mpz_init(foo);
 	try
 	{
-		// Check whether $a < p$.
-		if (mpz_cmp(a, p) >= 0)
-			throw false;
-		
-		// Check whether $a$ is not equal to zero.
-		if (!mpz_cmp_ui(a, 0L))
+		// Check whether $0 < a < p$.
+		if ((mpz_cmp_ui(a, 0L) <= 0) || (mpz_cmp(a, p) >= 0))
 			throw false;
 		
 		// Check whether $a^q \equiv 1 \pmod{p}$.
@@ -184,7 +180,8 @@ void BarnettSmartVTMF_dlog::RandomElement
 {
 	mpz_t b;
 	
-	// Choose randomly and uniformly an element $b$ from $\mathbb{Z}_q$.
+	// Choose randomly and uniformly an element $b$ from
+	// $\mathbb{Z}_q \setminus \{ 0 \}$.
 	mpz_init(b);
 	do
 		mpz_srandomm(b, q);
@@ -260,7 +257,7 @@ bool BarnettSmartVTMF_dlog::KeyGenerationProtocol_UpdateKey
 			throw false;
 		
 		// check the size of $r$
-		if (mpz_cmp(r, q) >= 0)
+		if (mpz_cmpabs(r, q) >= 0)
 			throw false;
 		
 		// verify the proof of knowledge [CaS97]
@@ -314,7 +311,8 @@ bool BarnettSmartVTMF_dlog::KeyGenerationProtocol_RemoveKey
 		if (h_j.find(fp.str()) != h_j.end())
 		{
 			// update the global key
-			assert(mpz_invert(foo, h_j[fp.str()], p));
+			if (!mpz_invert(foo, h_j[fp.str()], p))
+				throw false;
 			mpz_invert(foo, h_j[fp.str()], p);
 			mpz_mul(h, h, foo);
 			mpz_mod(h, h, p);
@@ -395,13 +393,14 @@ bool BarnettSmartVTMF_dlog::CP_Verify
 	try
 	{
 		// check the size of $r$
-		if (mpz_cmp(r, q) >= 0)
+		if (mpz_cmpabs(r, q) >= 0)
 			throw false;
 		
 		// verify proof of knowledge (equality of discrete logarithms) [CaS97]
 		if (fpowm_usage)
 		{
-			assert(!mpz_cmp(g, gg));
+			if (!mpz_cmp(g, gg))
+				throw false;
 			mpz_fpowm(fpowm_table_g, a, gg, r, p);
 		}
 		else
@@ -411,7 +410,8 @@ bool BarnettSmartVTMF_dlog::CP_Verify
 		mpz_mod(a, a, p);
 		if (fpowm_usage)
 		{
-			assert(!mpz_cmp(h, hh));
+			if (!mpz_cmp(h, hh))
+				throw false;
 			mpz_fpowm(fpowm_table_h, b, hh, r, p);
 		}
 		else
@@ -522,7 +522,7 @@ bool BarnettSmartVTMF_dlog::OR_Verify
 	try
 	{
 		// check the size of $r_1$ and $r_2$
-		if ((mpz_cmp(r_1, q) >= 0L) || (mpz_cmp(r_2, q) >= 0L))
+		if ((mpz_cmpabs(r_1, q) >= 0L) || (mpz_cmpabs(r_2, q) >= 0L))
 			throw false;
 		
 		// verify (S)PK ($y_1 = g_1^\alpha \vee y_2 = g_2^\beta$) [CaS97]
@@ -555,7 +555,8 @@ bool BarnettSmartVTMF_dlog::OR_Verify
 void BarnettSmartVTMF_dlog::MaskingValue
 	(mpz_ptr r) const
 {
-	// Choose randomly and uniformly an element from $\mathbb{Z}_q$.
+	// Choose randomly and uniformly an element from
+	// $\mathbb{Z}_q \setminus \{0, 1\}$.
 	do
 		mpz_srandomm(r, q);
 	while (!mpz_cmp_ui(r, 0L) || !mpz_cmp_ui(r, 1L));
