@@ -186,7 +186,7 @@ bool PedersenCommitmentScheme::CheckGroup
 			!mpz_probab_prime_p(q, TMCG_MR_ITERATIONS))
 				throw false;
 		
-		// Check whether $k$ is not a divisible by $q$, i.e. $q, k$ are coprime.
+		// Check whether $k$ is not divisible by $q$, i.e. $q, k$ are coprime.
 		mpz_gcd(foo, q, k);
 		if (mpz_cmp_ui(foo, 1L))
 			throw false;
@@ -284,6 +284,15 @@ void PedersenCommitmentScheme::CommitBy
 	mpz_clear(tmp);
 }
 
+bool PedersenCommitmentScheme::TestMembership
+	(mpz_srcptr c) const
+{
+	if ((mpz_cmp_ui(c, 0L) > 0) && (mpz_cmp(c, p) < 0))
+		return true;
+	else
+		return false;
+}
+
 bool PedersenCommitmentScheme::Verify
 	(mpz_srcptr c, mpz_srcptr r, const std::vector<mpz_ptr> &m) const
 {
@@ -303,7 +312,7 @@ bool PedersenCommitmentScheme::Verify
 		}
 		
 		// Verify the commitment: 1. $c\in\mathbb{Z}_p$ and 2. $c = c'$
-		if ((mpz_cmp(c, p) >= 1) || mpz_cmp(c, c2))
+		if ((mpz_cmp_ui(c, 0L) < 0) || (mpz_cmp(c, p) >= 0) || mpz_cmp(c, c2))
 			throw false;
 		
 		throw true;
@@ -403,23 +412,25 @@ void GrothSKC::Prove_interactive
 	}
 	
 	// prover: first move
-	in >> x;
+	in >> x; // get $x$ from the verifier
 	// check whether $x$ is from $\{0, 1\}^{\ell_e}$, otherwise reduce
 	if (mpz_sizeinbase(x, 2L) > l_e)
 		mpz_mod(x, x, exp2l_e);
 	
 	// prover: second move
-	mpz_srandomm(r_d, com->q);
-	mpz_srandomm(r_Delta, com->q);
+	mpz_srandomm(r_d, com->q); // $r_d \gets \mathbb{Z}_q$
+	mpz_srandomm(r_Delta, com->q); // $r_{\Delta} \gets \mathbb{Z}_q$
 	for (size_t i = 0; i < d.size(); i++)
-		mpz_srandomm(d[i], com->q);
-	mpz_set(Delta[0], d[0]);
+		mpz_srandomm(d[i], com->q); // $d_1,\ldots,d_n \gets \mathbb{Z}_q$
+	mpz_set(Delta[0], d[0]); // $\Delta_1 := d_1$
 	for (size_t i = 1; i < (Delta.size() - 1); i++)
-		mpz_srandomm(Delta[i], com->q);
-	mpz_set_ui(Delta[Delta.size() - 1], 0L);
+		mpz_srandomm(Delta[i], com->q);	// $\Delta_2,\ldots,\Delta_{n-1}
+										//           \gets \mathbb{Z}_q$
+	mpz_set_ui(Delta[Delta.size() - 1], 0L); // $\Delta_n := 0$
 	for (size_t i = 0; i < a.size(); i++)
 	{
 		mpz_set_ui(a[i], 1L);
+		// compute a_i = \prod_{j=1}^i (m_{\pi(j)} - x)
 		for (size_t j = 0; j <= i; j++)
 		{
 			mpz_sub(foo, m[pi[j]], x);
@@ -428,7 +439,8 @@ void GrothSKC::Prove_interactive
 			mpz_mod(a[i], a[i], com->q);
 		}
 	}
-	mpz_srandomm(r_a, com->q);
+	mpz_srandomm(r_a, com->q); // $r_a \gets \mathbb{Z}_q$
+	// $c_d = \mathrm{com}_{ck}(d_1,\ldots,d_n;r_d)$
 	com->CommitBy(c_d, r_d, d);
 	for (size_t i = 0; i < lej.size(); i++)
 	{
@@ -442,6 +454,8 @@ void GrothSKC::Prove_interactive
 		else
 			mpz_set_ui(lej[i], 0L);
 	}
+	// $c_{\Delta} = \mathrm{com}_{ck}(-\Delta_1 d_2,\ldots,
+	//                                 -\Delta_{n-1} d_n;r_{\Delta})$
 	com->CommitBy(c_Delta, r_Delta, lej);
 	for (size_t i = 0; i < lej.size(); i++)
 	{
@@ -463,16 +477,21 @@ void GrothSKC::Prove_interactive
 		else
 			mpz_set_ui(lej[i], 0L);
 	}
+	// $c_a = \mathrm{com}_{ck}(\Delta_2 - (m_{\pi(2)} - x)\Delta_1 - a_1 d_2,
+	//                          \ldots,\Delta_n - (m_{\pi(n)} - x)\Delta_{n-1}
+	//                                  - a_{n-1} d_n;r_a)$
 	com->CommitBy(c_a, r_a, lej);
+	// send $c_d$, $c_\Delta$, and $c_a$ to the verifier
 	out << c_d << std::endl << c_Delta << std::endl << c_a << std::endl;
 	
 	// prover: third move
-	in >> e;
+	in >> e; // get $e$ from the verifier
 	// check whether $x$ is from $\{0, 1\}^{\ell_e}$, otherwise reduce
 	if (mpz_sizeinbase(e, 2L) > l_e)
 		mpz_mod(e, e, exp2l_e);
 	
 	// prover: fourth move
+	// compute $f_i = e m_{\pi(i)} + d_i$
 	for (size_t i = 0; i < f.size(); i++)
 	{
 		mpz_mul(f[i], e, m[pi[i]]);
@@ -480,10 +499,13 @@ void GrothSKC::Prove_interactive
 		mpz_add(f[i], f[i], d[i]);
 		mpz_mod(f[i], f[i], com->q);
 	}
+	// compute $z = e r + r_d$
 	mpz_mul(z, e, r);
 	mpz_mod(z, z, com->q);
 	mpz_add(z, z, r_d);
 	mpz_mod(z, z, com->q);
+	// compute $f_{\Delta_i} = e (\Delta_{i+1} - (m_{\pi(i+1)} - x)\Delta_i
+	//                            - a_i d_{i+1}) - \Delta_i d_{i+1}$
 	for (size_t i = 0; i < (f_Delta.size() - 1); i++)
 	{
 		mpz_set(foo, Delta[i + 1]);
@@ -505,16 +527,18 @@ void GrothSKC::Prove_interactive
 		mpz_mod(foo, foo, com->q);
 		mpz_set(f_Delta[i], foo);
 	}
+	// compute $z_{\Delta} = e r_a + r_{\Delta}$
 	mpz_mul(z_Delta, e, r_a);
 	mpz_mod(z_Delta, z_Delta, com->q);
 	mpz_add(z_Delta, z_Delta, r_Delta);
 	mpz_mod(z_Delta, z_Delta, com->q);
 	for (size_t i = 0; i < f.size(); i++)
-		out << f[i] << std::endl;
-	out << z << std::endl;
+		out << f[i] << std::endl; // send $f_1,\ldots,f_n$ to the verifier
+	out << z << std::endl; // send $z$ to the verifier
 	for (size_t i = 0; i < (f_Delta.size() - 1); i++)
-		out << f_Delta[i] << std::endl;
-	out << z_Delta << std::endl;
+		out << f_Delta[i] << std::endl; // send $f_{\Delta_1},\ldots,
+		                                //      f_{\Delta_{n-1}}$ to verifier
+	out << z_Delta << std::endl; // send $z_{\Delta}$ to the verifier
 	
 	// release
 	mpz_clear(x), mpz_clear(r_d), mpz_clear(r_Delta), mpz_clear(r_a),
@@ -522,12 +546,13 @@ void GrothSKC::Prove_interactive
 		mpz_clear(z), mpz_clear(z_Delta), mpz_clear(foo), mpz_clear(bar);
 	for (size_t i = 0; i < m.size(); i++)
 	{
-		mpz_clear(d[i]), mpz_clear(Delta[i]), mpz_clear(a[i]), mpz_clear(f[i]), 
-			mpz_clear(f_Delta[i]), mpz_clear(lej[i]);
-		delete d[i], delete Delta[i], delete a[i], delete f[i], delete f_Delta[i], 
-			delete lej[i];
+		mpz_clear(d[i]), mpz_clear(Delta[i]), mpz_clear(a[i]), 
+			mpz_clear(f[i]), mpz_clear(f_Delta[i]), mpz_clear(lej[i]);
+		delete d[i], delete Delta[i], delete a[i], delete f[i], 
+			delete f_Delta[i], delete lej[i];
 	}
-	d.clear(), Delta.clear(), a.clear(), f.clear(), f_Delta.clear(), lej.clear();
+	d.clear(), Delta.clear(), a.clear(), f.clear(), f_Delta.clear(), 
+		lej.clear();
 }
 
 bool GrothSKC::Verify_interactive
@@ -555,27 +580,28 @@ bool GrothSKC::Verify_interactive
 	{
 		// verifier: first move
 		mpz_srandomb(x, l_e);
-		out << x << std::endl;
+		out << x << std::endl; // send $x\in\{0,1\}^{\ell_e}$ to the prover
 		
 		// verifier: second move
-		in >> c_d >> c_Delta >> c_a;
+		in >> c_d >> c_Delta >> c_a; // get $c_d$, $c_{\Delta}$, and $c_a$
+		                             // from the prover
 		
 		// verifier: third move
 		mpz_srandomb(e, l_e);
-		out << e << std::endl;
+		out << e << std::endl; // send $e\in\{0,1\}^{\ell_e}$ to prover
 		
 		// verifier: fourth move
 		for (size_t i = 0; i < f.size(); i++)
-			in >> f[i];
-		in >> z;
+			in >> f[i]; // get $f_1,\ldots,f_n$ from the prover
+		in >> z; // get $z$ from the prover
 		for (size_t i = 0; i < (f_Delta.size() - 1); i++)
-			in >> f_Delta[i];
-		in >> z_Delta;
+			in >> f_Delta[i]; // get $f_{\Delta_1},\ldots,f_{\Delta_{n-1}}$
+			                  // from the prover
+		in >> z_Delta; // get $z_{\Delta}$ from the prover
 		
-		// check whether $c_d, c_a, c_{\Delta} \in\mathcal{C}$
-		if (!(mpz_cmp(c_d, com->p) < 0) || !(mpz_cmp(c_a, com->p) < 0) ||
-			!(mpz_cmp(c_Delta, com->p) < 0) || !mpz_cmp_ui(c_d, 0L) ||
-			!mpz_cmp_ui(c_a, 0L) || !mpz_cmp_ui(c_Delta, 0L))
+		// check whether $c_d, c_a, c_{\Delta} \in\mathcal{C}_{ck}$
+		if (!(com->TestMembership(c_d) && com->TestMembership(c_a) &&
+			com->TestMembership(c_Delta)))
 				throw false;
 		
 		// check whether $f_1, \ldots, f_n, z \in\mathbb{Z}_q$
@@ -587,8 +613,8 @@ bool GrothSKC::Verify_interactive
 				throw false;
 		}
 		
-		// check whether $f_{\Delta_1}, \ldots, f_{\Delta_{n-1}}$ and $z$
-		// are from $\mathbb{Z}_q$
+		// check whether $f_{\Delta_1}, \ldots, f_{\Delta_{n-1}}$
+		// and $z_{\Delta}$ are from $\mathbb{Z}_q$
 		if (!(mpz_cmp(z_Delta, com->q) < 0))
 			throw false;
 		for (size_t i = 0; i < (f_Delta.size() - 1); i++)
@@ -599,7 +625,8 @@ bool GrothSKC::Verify_interactive
 		
 		if (optimizations)
 		{
-			// randomization technique from section 6, paragraph 'Batch verification'
+			// randomization technique from section 6,
+			// paragraph 'Batch verification' [Gr05]
 			mpz_t alpha;
 			mpz_init(alpha);
 			// pick $\alpha\in_R\{0, 1\}^{\ell_e}$ at random
@@ -635,14 +662,14 @@ bool GrothSKC::Verify_interactive
 		}
 		else
 		{
-			// check whether $c^e c_d = \mathrm{com}(f_1, \ldots, f_n; z)$
+			// check whether $c^e c_d = \mathrm{com}_{ck}(f_1,\ldots,f_n; z)$
 			mpz_powm(foo, c, e, com->p);
 			mpz_mul(foo, foo, c_d);
 			mpz_mod(foo, foo, com->p);
 			if (!com->Verify(foo, z, f))
 				throw false;
-			// check whether $c_a^e c_{\Delta} = \mathrm{com}(f_{\Delta_1},
-			// \ldots, f_{\Delta_{n-1}}; z_{Delta})$
+			// check whether $c_a^e c_{\Delta} = \mathrm{com}_{ck}
+			//            (f_{\Delta_1},\ldots,f_{\Delta_{n-1}}; z_{\Delta})$
 			mpz_powm(foo, c_a, e, com->p);
 			mpz_mul(foo, foo, c_Delta);
 			mpz_mod(foo, foo, com->p);
@@ -749,10 +776,9 @@ bool GrothSKC::Verify_interactive
 			in >> f_Delta[i];
 		in >> z_Delta;
 		
-		// check whether $c_d, c_a, c_{\Delta} \in\mathcal{C}$
-		if (!(mpz_cmp(c_d, com->p) < 0) || !(mpz_cmp(c_a, com->p) < 0) ||
-			!(mpz_cmp(c_Delta, com->p) < 0) || !mpz_cmp_ui(c_d, 0L) ||
-			!mpz_cmp_ui(c_a, 0L) || !mpz_cmp_ui(c_Delta, 0L))
+		// check whether $c_d, c_a, c_{\Delta} \in\mathcal{C}_{ck}$
+		if (!(com->TestMembership(c_d) && com->TestMembership(c_a) &&
+			com->TestMembership(c_Delta)))
 				throw false;
 		
 		// check whether $f_1, \ldots, f_n, z \in\mathbb{Z}_q$
@@ -764,8 +790,8 @@ bool GrothSKC::Verify_interactive
 				throw false;
 		}
 		
-		// check whether $f_{\Delta_1}, \ldots, f_{\Delta_{n-1}}$ and $z$
-		// are from $\mathbb{Z}_q$
+		// check whether $f_{\Delta_1}, \ldots, f_{\Delta_{n-1}}$
+		// and $z_{\Delta}$ are from $\mathbb{Z}_q$
 		if (!(mpz_cmp(z_Delta, com->q) < 0))
 			throw false;
 		for (size_t i = 0; i < (f_Delta.size() - 1); i++)
@@ -776,7 +802,8 @@ bool GrothSKC::Verify_interactive
 		
 		if (optimizations)
 		{
-			// randomization technique from section 6, paragraph 'Batch verification'
+			// randomization technique from section 6,
+			// paragraph 'Batch verification'
 			mpz_t alpha;
 			mpz_init(alpha);
 			// pick $\alpha\in_R\{0, 1\}^{\ell_e}$ at random
@@ -838,7 +865,7 @@ bool GrothSKC::Verify_interactive
 			if (!com->Verify(foo, z, lej))
 				throw false;
 			// check whether $c_a^e c_{\Delta} = \mathrm{com}(f_{\Delta_1},
-			// \ldots, f_{\Delta_{n-1}}; z_{Delta})$
+			// \ldots, f_{\Delta_{n-1}}; z_{\Delta})$
 			mpz_powm(foo, c_a, e, com->p);
 			mpz_mul(foo, foo, c_Delta);
 			mpz_mod(foo, foo, com->p);
@@ -1231,12 +1258,11 @@ bool GrothVSSHE::Verify_interactive
 		if (!skc->Verify_interactive(foo, f, m, in, out))
 			throw false;
 		
-		// check whether $c, c_d \in\mathcal{C}_{\mathrm{com}}$
-		if (!(mpz_cmp(c, com->p) < 0) || !(mpz_cmp(c_d, com->p) < 0) || 
-			!mpz_cmp_ui(c, 0L) || !mpz_cmp_ui(c_d, 0L))
-				throw false;
+		// check whether $c, c_d \in\mathcal{C}_{ck}$
+		if (!(com->TestMembership(c) && com->TestMembership(c_d)))
+			throw false;
 		
-		// check whether $E_d\in\mathcal{C}$
+		// check whether $E_d\in\mathcal{C}_{pk}$
 		mpz_fpowm(fpowm_table_g, foo, E_d.first, q, p);
 		mpz_fpowm(fpowm_table_h, bar, E_d.second, q, p);
 		if (mpz_cmp_ui(foo, 1L) || mpz_cmp_ui(bar, 1L))
@@ -1250,8 +1276,8 @@ bool GrothVSSHE::Verify_interactive
 					throw false;
 		}
 		
-		// check whether $Z\in\mathcal{R}$
-		if (mpz_cmp(Z, q) >= 0)
+		// check whether $Z\in\mathcal{R}_{pk}$
+		if ((mpz_cmp_ui(Z, 0L) <= 0) || (mpz_cmp(Z, q) >= 0))
 			throw false;
 		
 		// check whether
